/* step5　*/
#include "dm6.h"
#define NUM 9

dWorldID world;  // 動力学の世界
dSpaceID space;  // 衝突検出用スペース
dGeomID  ground; // 地面
dJointGroupID contactgroup; // コンタクトグループ

typedef struct {
	dBodyID body; // ボディのID
	dGeomID geom; // ジオメトリのID
	double *pos; // x, y, z　[m]
	double *R;   // 回転行列 要素数4x3
	double r, m; // 半径 [r], 質量 [kg]
	float *color; // 色 r,g,b
} MyObject;

MyObject apple[NUM];

// コールバック関数
void nearCallback(void *data, dGeomID o1, dGeomID o2)
{
  static const int N = 10; // 接触点数の最大値
  dContact contact[N];     // 接触点

  // 接触している物体のどちらかが地面ならisGroundに非0をセット
  int isGround = ((ground == o1) || (ground == o2));

  // 衝突情報の生成 nは衝突点数
  int n = dCollide(o1,o2,N,&contact[0].geom,sizeof(dContact));
  if (isGround)  {
     for (int i = 0; i < n; i++) {
       contact[i].surface.mode = dContactBounce; // 接触面の反発性を設定
       contact[i].surface.bounce = 1.0;          // 反発係数(0.0から1.0)
       contact[i].surface.bounce_vel = 0.0;      // 反発に必要な最低速度

       // 接触ジョイントの生成
       dJointID c = dJointCreateContact(world,contactgroup,
                                                &contact[i]);
       // 接触している２つの剛体を接触ジョイントにより拘束
       dJointAttach(c,dGeomGetBody(contact[i].geom.g1),
                      dGeomGetBody(contact[i].geom.g2));
    }
  }
}

void dmDraw(MyObject obj) /*** 物体の描画 ***/
{
	dsSetColor(obj.color[0],obj.color[1],obj.color[2]);  // 色の設定(r,g,b)
  const double *pos = dBodyGetPosition(obj.body);  // 位置を取得
  const double *R   = dBodyGetRotation(obj.body);  // 姿勢を取得
  dsDrawSphere(pos,R,obj.r);                       // 球の描画
}

void simLoop(int pause)           /***  シミュレーションループ　***/
{
  int i;

  dSpaceCollide(space,0,&nearCallback); // 衝突検出関数
  dWorldStep(world,0.01); // シミュレーションを１ステップ進める
  dJointGroupEmpty(contactgroup); // ジョイントグループを空にする

	for (i = 0; i < NUM; i++) {
		dmDraw(apple[i]); //  物体の描画
	}
}

void dmSphereCreate(MyObject *obj,double p[3], double R[12],
	double m, double r, float *color)
{
	obj->m = m;
	obj->r = r;
	obj->pos = p;
	obj->R   = R;
	obj->color = color;

	dRSetIdentity(obj->R);
	obj->body = dBodyCreate(world);           // ボールの生成
	dMass mass;                               // 構造体massの宣言
  dMassSetZero(&mass);                      // 構造体massの初期化
  dMassSetSphereTotal(&mass,obj->m,obj->r); // 構造体massに質量を設定
  dBodySetMass(obj->body,&mass);            // ボールにmassを設定
  dBodySetPosition(obj->body,obj->pos[0],obj->pos[1],obj->pos[2]); // ボールの位置(x,y,z)を設定

  obj->geom = dCreateSphere(space,obj->r); // 球ジオメトリの生成
  dGeomSetBody(obj->geom, obj->body);      // ボディとジオメトリの関連付け
}

int main()         /*** main関数 ***/
{
  // リンゴの変数
  int i;
	double p[3] = {0.0, -2.5, 2.0}; // 位置
	double R[12];   // 回転行列
	double m = 1.0; // 質量
	double r = 0.2; // 半径
	float  color[3] = {1.0, 0.0, 0.0}; // 色

	dInitODE(); // ODEの初期化
  world = dWorldCreate();               // 動力学用世界の創造
  space        = dHashSpaceCreate(0);   // 衝突用空間の創造
  contactgroup = dJointGroupCreate(0);  // ジョイントグループの生成
  ground = dCreatePlane(space,0,0,1,0); // 地面（平面ジオメトリ）の生成
  dWorldSetGravity(world,0,0,-0.2);     // 重力設定

  dRSetIdentity(R);  //回転行列を単位行列で初期化

  for (i = 0; i < NUM; i++) {
  	p[1] += 0.5;
		dmSphereCreate(&apple[i],p,R,m,r,color); // リンゴの生成
  }

  dmLoop(800, 600);  // シミュレーションループ ウインドウの幅，高

  dSpaceDestroy(space); // 衝突用空間の破壊
	dWorldDestroy(world); // 動力学用世界の破壊
	dCloseODE();          // ODEの終了
  return 0;
}




